@@ -8,7 +8,7 @@
  ***************************************************************************/
 
  static char *const _id =
-"$Id: getqueue.c,v 1.42 2002/12/07 00:30:37 papowell Exp $";
+"$Id: getqueue.c,v 1.48 2003/04/15 23:37:42 papowell Exp $";
 
 
 /***************************************************************************
@@ -58,13 +58,16 @@
 
 int Scan_queue( struct line_list *spool_control,
 	struct line_list *sort_order, int *pprintable, int *pheld, int *pmove,
-		int only_queue_process, int *perr, int *pdone )
+		int only_queue_process, int *perr, int *pdone,
+		const char *remove_prefix, const char *remove_suffix )
 {
 	DIR *dir;						/* directory */
 	struct dirent *d;				/* directory entry */
 	struct line_list directory_files;
 	char *hf_name;
 	int c, printable, held, move, error, done, p, h, m, e, dn;
+	int remove_prefix_len = safestrlen( remove_prefix );
+	int remove_suffix_len = safestrlen( remove_suffix );
 	struct job job;
 
 	c = printable = held = move = error = done = 0;
@@ -84,22 +87,27 @@ int Scan_queue( struct line_list *spool_control,
 
 	hf_name = 0;
 	while(1){
-		hf_name = 0;
-		while( hf_name == 0 && (d = readdir(dir)) ){
+		while( (d = readdir(dir)) ){
 			hf_name = d->d_name;
 			DEBUG5("Scan_queue: found file '%s'", hf_name );
-			if(    (cval(hf_name+0) == 'h')
+			if(
+				(remove_prefix_len && !strncmp( hf_name, remove_prefix, remove_prefix_len ) )
+				|| (remove_suffix_len 
+					&& !strcmp( hf_name+strlen(hf_name)-remove_suffix_len, remove_suffix ))
+			){
+				DEBUG1("Scan_queue: removing file '%s'", hf_name );
+				unlink( hf_name );
+				continue;
+			} else if(    (cval(hf_name+0) == 'h')
 				&& (cval(hf_name+1) == 'f')
 				&& isalpha(cval(hf_name+2))
 				&& isdigit(cval(hf_name+3))
 				){
 				break;
-			} else {
-				hf_name = 0;
 			}
 		}
 		/* found them all */
-		if( hf_name == 0 ){
+		if( d == 0 ){
 			break;
 		}
 
@@ -160,16 +168,20 @@ int Scan_queue( struct line_list *spool_control,
  *  Get an image of a file from an fd
  */
 
-char *Get_fd_image( int fd, int maxsize )
+char *Get_fd_image( int fd, off_t maxsize )
 {
 	char *s = 0;
 	struct stat statb;
 	char buffer[LARGEBUFFER];
-	int n, len;
+	int n;
+	off_t len;
 
 	DEBUG3("Get_fd_image: fd %d", fd );
 
-	lseek(fd, 0, SEEK_SET);
+	if( lseek(fd, 0, SEEK_SET) == -1 ){
+		Errorcode = JFAIL;
+		LOGERR_DIE(LOG_INFO)"Get_fd_image: lseek failed" );
+	}
 	if( maxsize && fstat(fd, &statb) == 0
 		&& maxsize< statb.st_size/1024 ){
 		lseek(fd, -maxsize*1024, SEEK_END);
@@ -194,7 +206,7 @@ char *Get_fd_image( int fd, int maxsize )
  *  Get an image of a file
  */
 
-char *Get_file_image( const char *file, int maxsize )
+char *Get_file_image( const char *file, off_t maxsize )
 {
 	char *s = 0;
 	struct stat statb;
@@ -1402,6 +1414,7 @@ void Job_printable( struct job *job, struct line_list *spool_control,
 	char *s;
 	char buffer[SMALLBUFFER];
 	char destbuffer[SMALLBUFFER];
+	struct stat statb; 
 	int n, printable = 0, held = 0, move = 0, error = 0, done = 0,destination, destinations;
 
 	if(DEBUGL4)Dump_job("Job_printable - job info",job);
@@ -1422,8 +1435,17 @@ void Job_printable( struct job *job, struct line_list *spool_control,
 	} else if( (n = Find_flag_value(&job->info,SERVER,Value_sep))
 		&& kill( n, 0 ) == 0 ){
 		int delta;
-		n = Find_flag_value(&job->info,START_TIME,Value_sep);
-		delta = time((void *)0) - n;
+		time_t now = time((void *)0);
+		time_t last_change = Find_flag_value(&job->info,START_TIME,Value_sep);
+		if( !ISNULL(Status_file_DYN) && !stat( Status_file_DYN, &statb )
+			&& last_change < statb.st_mtime ){
+			last_change = statb.st_mtime;
+		}
+		if( !ISNULL(Log_file_DYN) && !stat( Log_file_DYN, &statb )
+			&& last_change < statb.st_mtime ){
+			last_change = statb.st_mtime;
+		}
+		delta = now - last_change;
 		if( Stalled_time_DYN && delta > Stalled_time_DYN ){
 			SNPRINTF( buffer, sizeof(buffer))
 				"stalled(%dsec)", delta );
@@ -1872,6 +1894,8 @@ void Fix_control( struct job *job, char *filter, char *xlate_format )
 		s = job->info.list[i];
 		if( s && (c = s[0]) && isupper(c) && c != 'N' && c != 'U'
 			&& s[1] == '=' ){
+			/* remove banner from control file */
+			if( c == 'L' && Suppress_header_DYN && !Always_banner_DYN ) continue;
 			s[1] = 0;
 			Add_line_list( &controlfile, s, Value_sep, 1, 1 );
 			Set_str_value(&controlfile,s,s+2);
