@@ -8,7 +8,7 @@
  ***************************************************************************/
 
  static char *const _id =
-"$Id: vars.c,v 1.42 2002/12/07 00:30:39 papowell Exp $";
+"$Id: vars.c,v 1.48 2003/04/15 23:37:42 papowell Exp $";
 
 
 /* force local definitions */
@@ -210,6 +210,8 @@ struct keywords Pc_var_list[] = {
 { "fakelargefile", 0,  INTEGER_K,  &Fake_large_file_DYN,0,0,0},
    /*  string to send for a form feed */
 { "ff", 0,  STRING_K,  &Form_feed_DYN,0,0,"=\\f"},
+   /*  string to send for a form feed */
+{ "ff_separator", 0,  STRING_K,  &FF_separator_DYN,0,0,0},
    /* enforce FIFO (first in, first out) order */
 { "fifo", 0, STRING_K, &Fifo_DYN,0,0,0},
    /* FIFO lock file */
@@ -305,9 +307,9 @@ struct keywords Pc_var_list[] = {
    /*  lpd pathname for server use */
 { "lpd_path", 0,  STRING_K,  &Lpd_path_DYN,0,0,0},
    /* max number of queues to start at a time */
-{ "lpd_poll_servers_started", 0, INTEGER_K, &Poll_servers_started_DYN,1,0,"=10"},
+{ "lpd_poll_servers_started", 0, INTEGER_K, &Poll_servers_started_DYN,1,0,"=3"},
    /*  interval in secs between starting small numbers of queue servers */
-{ "lpd_poll_start_interval", 0,  INTEGER_K,  &Poll_start_interval_DYN,0,0,"=10"},
+{ "lpd_poll_start_interval", 0,  INTEGER_K,  &Poll_start_interval_DYN,0,0,"=1"},
    /*  interval in secs between servicing all queues */
 { "lpd_poll_time", 0,  INTEGER_K,  &Poll_time_DYN,0,0,"=600"},
    /* lpd port */
@@ -460,8 +462,8 @@ struct keywords Pc_var_list[] = {
 { "send_data_first", 0, FLAG_K, &Send_data_first_DYN,0,0,0},
    /* failure action for server to take after send_try attempts failed */
 { "send_failure_action", 0, STRING_K, &Send_failure_action_DYN,0,0,"=remove"},
-   /* timeout for read/write lpr IO operatons (10 min default) */
-{ "send_job_rw_timeout", 0, INTEGER_K, &Send_job_rw_timeout_DYN,0,0,"=600"},
+   /* timeout for status or job file completion by filter or printer (default 0 - never time out) */
+{ "send_job_rw_timeout", 0, INTEGER_K, &Send_job_rw_timeout_DYN,0,0,"=0"},
    /* timeout for read/write status or control operatons */
 { "send_query_rw_timeout", 0, INTEGER_K, &Send_query_rw_timeout_DYN,0,0,"=30"},
    /* numbers of times for server to try sending job - 0 is infinite */
